#! /usr/bin/env python

import os
import subprocess
import sys

from distutils.cmd import Command
from distutils.command.build import build as _build
from distutils.command.sdist import sdist
from setuptools.command.install_lib import install_lib as _install_lib
from setuptools import setup, find_packages

local_cfg = None
if os.path.exists('wcs/wcs_cfg.py'):
    local_cfg = file('wcs/wcs_cfg.py').read()
    os.unlink('wcs/wcs_cfg.py')


class compile_translations(Command):
    description = 'compile message catalogs to MO files via django compilemessages'
    user_options = []

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        try:
            from django.core.management import call_command
            for path, dirs, files in os.walk('wcs'):
                if 'locale' not in dirs:
                    continue
                curdir = os.getcwd()
                os.chdir(os.path.realpath(path))
                call_command('compilemessages')
                os.chdir(curdir)
        except ImportError:
            sys.stderr.write('!!! Please install Django >= 1.4 to build translations\n')


class build(_build):
    sub_commands = [('compile_translations', None)] + _build.sub_commands


class install_lib(_install_lib):
    def run(self):
        self.run_command('compile_translations')
        _install_lib.run(self)


class eo_sdist(sdist):

    def run(self):
        print "creating VERSION file"
        if os.path.exists('VERSION'):
            os.remove('VERSION')
        version = get_version()
        version_file = open('VERSION', 'w')
        version_file.write(version)
        version_file.close()
        sdist.run(self)
        print "removing VERSION file"
        if os.path.exists('VERSION'):
            os.remove('VERSION')


def data_tree(destdir, sourcedir):
    extensions = ['.css', '.png', '.jpeg', '.jpg', '.gif', '.xml', '.html',
            '.js', '.ezt', '.dat', '.eot', '.svg', '.ttf', '.woff']
    r = []
    for root, dirs, files in os.walk(sourcedir):
        l = [os.path.join(root, x) for x in files if os.path.splitext(x)[1] in extensions]
        r.append( (root.replace(sourcedir, destdir, 1), l) )
        for vcs_dirname in ('CVS', '.svn', '.bzr', '.git'):
            if vcs_dirname in dirs:
                dirs.remove(vcs_dirname)
    return r

def get_version():
    if os.path.exists('VERSION'):
        version_file = open('VERSION', 'r')
        version = version_file.read()
        version_file.close()
        return version
    if os.path.exists('.git'):
        p = subprocess.Popen(['git','describe','--match=v*'], stdout=subprocess.PIPE)
        result = p.communicate()[0]
        version = result.split()[0][1:]
        version = version.replace('-', '.')
        return version
    return '0'


cmdclass = {
    'build': build,
    'compile_translations': compile_translations,
    'install_lib': install_lib,
    'sdist': eo_sdist
}

setup(
        name = 'wcs',
        version = get_version(),
        maintainer = "Frederic Peters",
        maintainer_email = "fpeters@entrouvert.com",
        url = "http://wcs.labs.libre-entreprise.org",
        install_requires=['gadjo'],
        package_dir = { 'wcs': 'wcs' },
        packages = find_packages(),
        cmdclass = cmdclass,
        scripts = ['wcsctl.py', 'manage.py'],
        include_package_data=True,
        data_files = data_tree('share/wcs/web/', 'data/web/') + \
                data_tree('share/wcs/themes/', 'data/themes/') + \
                data_tree('share/wcs/vendor/', 'data/vendor/') + \
                data_tree('share/wcs/qommon/', 'wcs/qommon/static/') +
                [('share/wcs/', ['data/webbots', 'data/print-html-as-pdf.js'])]
    )

if local_cfg:
    file('wcs/wcs_cfg.py', 'w').write(local_cfg)
