# w.c.s. - web application for online forms
# Copyright (C) 2005-2013  Entr'ouvert
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>.

import tarfile
import time
import os

from qommon.ctl import Command, make_option


class CmdBackup(Command):
    name = 'backup'

    def __init__(self):
        Command.__init__(self, [
                make_option('--file', metavar='FILENAME', action='store',
                            dest='filename', default=None)
                ])

    def execute(self, base_options, sub_options, args):
        import publisher

        publisher.WcsPublisher.configure(self.config)
        pub = publisher.WcsPublisher.create_publisher(
                register_tld_names=False)

        hostname = args[0]
        pub.app_dir = os.path.join(pub.app_dir, hostname)
        if not os.path.exists(pub.app_dir):
            return 1

        if sub_options.filename:
            backup_filepath = sub_options.filename
        else:
            backup_dir = os.path.join(pub.app_dir, 'backups')
            if not os.path.exists(backup_dir):
                os.mkdir(backup_dir)
            backup_filepath = os.path.join(backup_dir,
                            'backup-%s%s%s-%s%s%s.tar.gz' % time.localtime()[:6])

        backup = tarfile.open(backup_filepath, mode='w:gz')
        for basename, dirnames, filenames in os.walk(pub.app_dir):
            if 'backups' in dirnames: # do not recurse in backup directory
                idx = dirnames.index('backups')
                dirnames[idx:idx+1] = []
            for filename in filenames:
                backup.add(os.path.join(basename, filename),
                           os.path.join(basename, filename)[len(pub.app_dir):])

        backup.close()


CmdBackup.register()
